#
# Copyright (C) 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from managesf.lib.auth.cauth import CAuthAuthenticator
from managesf.lib.auth.keycloak import KeycloakAuthenticator


log = logging.getLogger(__name__)


def get_auth_engine(services):
    try:
        if 'keycloak' in services:
            return KeycloakAuthenticator()
        else:
            return CAuthAuthenticator()
    except Exception as e:
        log.info(
            'An exception occurred while getting '
            'the authentication engine: "%s"; defaulting to cauth' % e)
        return CAuthAuthenticator()
