#
# Copyright (C) 2017 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import logging

from managesf.api.v2 import base


logger = logging.getLogger(__name__)


resource_manager = None
logger.info('Loading resources manager...')
try:
    resource_manager = base.load_manager('managesf.v2.resources',
                                         'manageSF')
except Exception as e:
    msg = 'Cannot load resource service: %s' % e
    logger.error(msg)
    raise
